#ifndef ErrorModel80211_H
#define ErrorModel80211_H

const int EM80211_MAX_SNR=50;
const int EM80211_NUM_MOD=4;

const int EM80211_BPSK=0;
const int EM80211_QPSK=1;
const int EM80211_CCK55=2;
const int EM80211_CCK11=3;

const int EM80211_LPREAMBLE_L=144;
const int EM80211_SPREAMBLE_L=72;
const int EM80211_PLCP_L=48;

const int EM80211_DATA = 0;
const int EM80211_ACK = 1;
const int EM80211_RTS = 2;
const int EM80211_CTS = 3;

const int EM80211_ACK_L=14*8;		/*MAC header included*/
const int EM80211_RTS_L=20*8;		/*MAC header included*/
const int EM80211_CTS_L=14*8;		/*MAC_header_included*/

class ErrorModel80211 : public TclObject
{
public:
	
	/*initialize from one ber_snr file*/
	static int initialize(const char* const& filename);

	static double ferBySnr(int frametype, int snr, int len, double rate);

	static int ErrorModel80211::snrDeltaBetweenRates(double rate1, double rate2);	

	/*Return frame error rate and SNR by snr*/
	static double frameErrorRate(int frametype, double signal,double interference,double rate, int fullLength, int* snr_return);
	static double frameErrorRate(double ber, int pkt_len);

	static int initialized_;
        static int isShortPreamble_;

        static int noise11_;    /*unit is dBm*/
        static int noise55_;
        static int noise1_;
        static int noise2_;

        static double noiseW11_;    /*unit is Watt*/
        static double noiseW55_;
        static double noiseW1_;
        static double noiseW2_;


private:

	static void printTables();
	static void preCalculate();

        static double ber_snr_[EM80211_NUM_MOD][EM80211_MAX_SNR+1];
        static double preamble_succProb_[EM80211_MAX_SNR+1];
        static double plcp_succProb_[EM80211_MAX_SNR+1];
        static double ack_succProb_[EM80211_NUM_MOD][EM80211_MAX_SNR+1];
        static double rts_succProb_[EM80211_NUM_MOD][EM80211_MAX_SNR+1];
        static double cts_succProb_[EM80211_NUM_MOD][EM80211_MAX_SNR+1];

};

#endif

