#ifndef lint
static const char rcsid[] =
    "@(#) $Header: /nfs/jade/vint/CVSROOT/ns-2/queue/drop-tail.cc,v 1.17 2004/10/28 23:35:37 haldar Exp $ (LBL)";
#endif

#include "Pri_dtail.h"

static class Pri_dtailClass : public TclClass {
 public:
	Pri_dtailClass() : TclClass("Queue/Pri_dtail") {}
	TclObject* create(int, const char*const*) {
		return (new Pri_dtail);
	}
} class_drop_tail;

void Pri_dtail::reset()
{
	Queue::reset();
}

int 
Pri_dtail::command(int argc, const char*const* argv) 
{
	if (argc==2) {
		if (strcmp(argv[1], "printstats") == 0) {
			print_summarystats();
			return (TCL_OK);
		}
 		if (strcmp(argv[1], "shrink-queue") == 0) {
 			shrink_queue();
 			return (TCL_OK);
 		}
	}
	if (argc == 3) {
		if (!strcmp(argv[1], "packetqueue-attach")) {
			delete q_;
			if (!(q_ = (PacketQueue*) TclObject::lookup(argv[2])))
				return (TCL_ERROR);
			else {
				pq_ = q_;
				return (TCL_OK);
			}
		}
	}
	return Queue::command(argc, argv);
}

/*
 * drop-tail
 */
 
bool Pri_dtail::isAdd2_Overflow(Packet * p) { // check if it will overflow when 2 codable packets are queued
	int qlimBytes = qlim_ * mean_pktsize_;
	if ((!qib_ && (q_->length() + 2) >= qlim_) ||
  		(qib_ && (q_->byteLength() + 2 * hdr_cmn::access(p)->size()) >= qlimBytes)){
  		return true;
  	} else
  		return false;
}
 
void Pri_dtail::enqueHead(Packet* p) { 
	if (summarystats) {
                Queue::updateStats(qib_?q_->byteLength():q_->length());
	}

	int qlimBytes = qlim_ * mean_pktsize_;
	if ((!qib_ && (q_->length() + 1) >= qlim_) ||
  	(qib_ && (q_->byteLength() + hdr_cmn::access(p)->size()) >= qlimBytes)){
		if (drop_front_)
			Packet::free(p);
		else {
  		  	q_->enqueHead(p);
	  		(*pkt_map_)[hdr_cmn::access(p)->uid()] = p;
  		  	Packet * tailp = q_->tail(); 
  		  	q_->remove(tailp, tailp->prev_);
  			
  			(*pkt_map_).erase(hdr_cmn::access(tailp)->uid());  		  	
  		  	Packet::free(tailp); 
		}
  	} else {
  		  	q_->enqueHead(p);
  		  	if (pkt_map_)
  		  		(*pkt_map_)[hdr_cmn::access(p)->uid()] = p;
  	}  		  
}

void Pri_dtail::enque(Packet* p)
{
	if (summarystats) {
                Queue::updateStats(qib_?q_->byteLength():q_->length());
	}

	int qlimBytes = qlim_ * mean_pktsize_;
	if ((!qib_ && (q_->length() + 1) >= qlim_) ||
  	(qib_ && (q_->byteLength() + hdr_cmn::access(p)->size()) >= qlimBytes)){
		// if the queue would overflow if we added this packet...
		if (drop_front_) { /* remove from head of queue */
			q_->enque(p);
			if (pkt_map_)
				(*pkt_map_)[hdr_cmn::access(p)->uid()] = p;
			//Packet *pp = q_->deque();
			//(*pkt_map_).erase(hdr_cmn::access(pp)->uid());
			Packet *pp = deque();
			Packet::free(pp);
		} else {
			// (*pkt_map_).erase(hdr_cmn::access(p)->uid());   !! not needed here. It is not dropped without being Qed
			Packet::free(p);
		}
	} else {
		q_->enque(p);
		if (pkt_map_)
			(*pkt_map_)[hdr_cmn::access(p)->uid()] = p;
	}
}

bool Pri_dtail::enque_insert(Packet* p, Packet* pp)
{
	if (summarystats) {
                Queue::updateStats(qib_?q_->byteLength():q_->length());
	}

	int qlimBytes = qlim_ * mean_pktsize_;
	if ((!qib_ && (q_->length() + 1) >= qlim_) ||
  	(qib_ && (q_->byteLength() + hdr_cmn::access(p)->size()) >= qlimBytes)){
		// if the queue would overflow if we added this packet...
		return false;
	} else {
			q_->insert(p, pp);
			if (pkt_map_)
				(*pkt_map_)[hdr_cmn::access(p)->uid()] = p;
			return true;
	}
}

//AG if queue size changes, we drop excessive packets...
void Pri_dtail::shrink_queue() 
{
        int qlimBytes = qlim_ * mean_pktsize_;
	if (debug_)
		printf("shrink-queue: time %5.2f qlen %d, qlim %d\n",
 			Scheduler::instance().clock(),
			q_->length(), qlim_);
        while ((!qib_ && q_->length() > qlim_) || 
            (qib_ && q_->byteLength() > qlimBytes)) {
                if (drop_front_) { /* remove from head of queue */
                        Packet *pp = q_->deque();
                        if (pkt_map_)
                        	(*pkt_map_).erase(hdr_cmn::access(pp)->uid());
                        Packet::free(pp);
                } else {
                        Packet *pp = q_->tail();
                        q_->remove(pp);
                       	(*pkt_map_).erase(hdr_cmn::access(pp)->uid());
                        Packet::free(pp);
                }
        }
}

Packet* Pri_dtail::deque()
{
        if (summarystats && &Scheduler::instance() != NULL) {
                Queue::updateStats(qib_?q_->byteLength():q_->length());
        }
  
  	Packet * p = q_->deque();
  	if (pkt_map_ && p)
    	(*pkt_map_).erase(hdr_cmn::access(p)->uid());
	return p;
}

void Pri_dtail::print_summarystats()
{
	//double now = Scheduler::instance().clock();
        printf("True average queue: %5.3f", true_ave_);
        if (qib_)
                printf(" (in bytes)");
        printf(" time: %5.3f\n", total_time_);
}

//~ void Pri_dtail::target_handle(NsObject* targ, Queue * hdl){
    //~ target_ = targ;
	//~ hdl_Psched_ = hdl;
//~ }

