#include <object.h>
#include <queue.h>
#include <packet.h>
#include <cmu-trace.h>
#include "Pri_sched.h"
#include "random.h"
#include "mac-802_11.h"

typedef int (*PacketFilter)(Packet *, void *);

Pri_sched_List Pri_sched::prhead = { 0 };

static class Pri_schedClass : public TclClass {
public:
  Pri_schedClass() : TclClass("Queue/Pri_dtail/Pri_sched") {}
  TclObject* create(int, const char*const*) {
    return (new Pri_sched);
  }
} class_Pri_sched;


Pri_sched::Pri_sched() : qlenTimer_(this), Pri_dtail()
{
    bind("Prefer_Routing_Protocols", &Prefer_Routing_Protocols);
	LIST_INSERT_HEAD(&prhead, this, link);
	cntl = 0;
	flag = 0;
	packet_left_ = 0;
	disable_coding = false;
	
	bind("qlen_trace_prd_", &qlen_trace_prd_);
	if (qlen_trace_prd_ > 0)
		qlenTimer_.resched(qlen_trace_prd_);
	
	Qbroadcast_.pkt_map_ = NULL;
	Qnative_.pkt_map_ = &(uid_pkt_map);
	Qoverheard_.pkt_map_ = &(uid_pkt_map);
	Qcodable_.pkt_map_ = &(uid_pkt_map);
	
	resetQcodeOnce();
}

int
Pri_sched::command(int argc, const char*const* argv)
{
    //Scheduler &s = Scheduler::instance();
    if (argc == 2) {
		if (strcasecmp(argv[1], "reset") == 0) 
			Terminate();
			//FALL-THROUGH to give parents a chance to reset
		else if (strcasecmp(argv[1], "disable-coding") == 0){
			disable_coding = true;
			return TCL_OK;
		}
    }

	if (argc == 3) {
		if (strcmp(argv[1], "qlen-trace") == 0) {
                        if (strcmp(argv[2], "") == 0) { return(TCL_OK); }
                        int mode;
                        const char* id = argv[2];
                        Tcl& tcl = Tcl::instance();
                        t_ch = Tcl_GetChannel(tcl.interp(), (char*)id, &mode);
                        return(TCL_OK);
		} else if(strcmp(argv[1], "arptable") == 0) {
          			 arptable_ = (ARPTable*)TclObject::lookup(argv[2]);
         			 assert(arptable_);
         			 return TCL_OK;
      	}
	}  else if (argc == 2) { 
		if(strcmp(argv[1], "debug-qlen-chk") == 0) {
   			int count = 0;
      		for (Packet *tmp = Qnative_.head(); tmp; tmp = tmp->next_)
      				count ++;
      		printf("\n NODE:%d  --IFQ \n", ((Mac802_11 *) target())->addr());
      		printf("Qnative_   Count: %d	 Length: %d\n", count, Qnative_.length());
      		
      		count = 0;
      		for (Packet *tmp = Qoverheard_.head(); tmp; tmp = tmp->next_)
      				count ++;
      		printf("Qoverheard_   Count: %d	 Length: %d\n", count, Qoverheard_.length());

      		count = 0;
      		for (Packet *tmp = Qcodable_.head(); tmp; tmp = tmp->next_)
      			count ++;
      		printf("\n Qcodable_   Count: %d	 Length: %d\n", count, Qcodable_.length());
      		printf("Map_size: %d\n", uid_pkt_map.size());
			return TCL_OK;
		}
		if(strcmp(argv[1], "debug-q-content") == 0) {
   			printf("\n NODE:%d  --Qnative_content \n", ((Mac802_11 *) target())->addr());
   			int count = 0;
      		for (Packet *tmp = Qnative_.head(); tmp; tmp = tmp->next_) {
      				count ++;
      				struct hdr_cmn *ch_p = HDR_CMN(tmp);
      				printf("#%03d uid-%d pcond-%d prev-%d next-%d 2hn-%d\n", 
      					count, ch_p->uid(), ch_p->pcondition, ch_p->prev_fwd_mac, ch_p->next_hop_, ch_p->two_hop_away_ip);
      		}
      		count = 0;
   			printf("\n NODE:%d  --Qcodable_content  \n", ((Mac802_11 *) target())->addr());      		
      		for (Packet *tmp = Qcodable_.head(); tmp; tmp = tmp->next_) {
      			count ++;
      				struct hdr_cmn *ch_p = HDR_CMN(tmp);
      				printf("#%03d uid-%d pcond-%d prev-%d next-%d 2hn-%d next-partnerID-%d\n", 
      					count, ch_p->uid(), ch_p->pcondition, ch_p->prev_fwd_mac, ch_p->next_hop_, 
      					ch_p->two_hop_away_ip, HDR_CMN(ch_p->partner_p)->uid());
      		}
			return TCL_OK;
		}
		
	}
	
    return Pri_dtail::command(argc, argv);
}

void
Pri_sched::recv(Packet *p, Handler *h)
{
    if(flag == 0) {
		((Mac802_11 *) target()) -> pp_sched_ = this;
		flag = 1;
	}
    
//// **** debug
//	int uidd = hdr_cmn::access(p)->uid();
//	int indx =((Mac802_11 *) target())->addr();
//	double now = Scheduler::instance().clock();
//	if (uidd == 41376 && indx == 6) {
//		printf(" ");
//	}//debug jz

    struct hdr_cmn *ch = HDR_CMN(p);
		
    if(Prefer_Routing_Protocols) {

                switch(ch->ptype()) {
                case PT_DSR:
                case PT_MESSAGE:
                case PT_TORA:
                case PT_AODV:
                case PT_ARP:
                        first_pri_recv(p, h);
                        break;
			//some of them (aodv) are not broadcasting, need to check it in MAC
                default:
		    			pri_recv(p, h);
                }
        }
    else {
	    pri_recv(p, h);
    }
}

void Pri_sched::first_pri_recv(Packet *p, Handler *h)
{
	Qbroadcast_.recv(p,h);  // only enque the packet
	hdr_cmn::access(p)->two_hop_away_ip = RESERVED_END_IP; // make sure ARP sent as native and not buffered or queued by others
	if (!blocked_)	sched();
}    

void Pri_sched::pri_recv(Packet *p, Handler *h)
{
//    Scheduler &s = Scheduler::instance();
    struct hdr_cmn *ch = HDR_CMN(p);
    
	struct hdr_mac802_11* dh = HDR_MAC802_11(p);
	u_int32_t mac_addr = (u_int32_t) ETHER_ADDR(dh->dh_ra);

	if (mac_addr == MAC_BROADCAST) {
		Qbroadcast_.recv(p,h);  // only enque the packet
	} else {
		assert(ch->size_ > CODING_SIZE_THRHLD && (ch->ptype() == PT_CBR));
		// change it later; if size < thrhld, only put in Qnative_;
		
		//search in map; if it's already there, print what is wrong
		//find the pointer and its qlocation. If it is codable, remove both it and 
		//its partner, and re-enque them.
		uid_pkt_tbl_t::iterator mp = uid_pkt_map.find(ch->uid());			
		if (mp != uid_pkt_map.end()) {
//		if (Packet *extra = removePktInQ(ch->uid())) {
			//printf("Found duplicated packet in map and queue");
			drop(p, "inQalready");
			if (!blocked_)	sched();
			return;
		}

//diversity control here
	
		if (ch->pcondition != PCOND_INTENDED_CODABLE && ch->pcondition != PCOND_INTENDED_UNCODABLE) { 
			int nhopCount = 0;
			for (Packet *pp = 0, *po = Qnative_.head(); po; pp= po, po= po->next_) {
				struct hdr_mac802_11* hpo = HDR_MAC802_11(po);
				u_int32_t nextHop = (u_int32_t) ETHER_ADDR(hpo->dh_ra);
				if (nextHop == mac_addr) nhopCount ++;
			}
			
			int nhopLim = Qnative_.limit()*DIVERSITY_FACTOR;
			if (nhopCount > nhopLim) {
				drop(p, "DIVSTY");
				if (!blocked_)	sched();
				return;
			}
		}
		
		if (!disable_coding)
	    	queue_and_searchPartner(p, true);
	    else {
	    	if (ch->pcondition == PCOND_OVERHEARD) {
	    		Qoverheard_.enque(p);
	    		hdr_cmn::access(p)->qlocation = QLOC_OVERHEARD;
	    	} else {
	    		Qnative_.enque(p);
				hdr_cmn::access(p)->qlocation = QLOC_NATIVE;
	    	}
	    }
	}
	
	if (!blocked_)	sched();
}    

bool Pri_sched::activeMacDrop(u_int32_t prev) {
	int nhopCount = 0;
	for (Packet *pp = 0, *po = Qnative_.head(); po; pp= po, po= po->next_) {
		u_int32_t prevHop = HDR_CMN(po)->prev_fwd_mac;
		if (prevHop == prev) nhopCount ++;
	}
//	for (Packet *pp = 0, *po = Qoverheard_.head(); po; pp= po, po= po->next_) {
//		u_int32_t prevHop = HDR_CMN(po)->prev_fwd_mac;
//		if (prevHop == prev) nhopCount ++;
//	}
	
	int nhopLim = Qnative_.limit()*DIVERSITY_FACTOR;
	if ( (nhopCount > nhopLim) && (Random::uniform(1.0) <= 0.2) ){
			return true;
	}
	return false;
}

bool Pri_sched::activeMacDrop_overheard(u_int32_t prev) {
	int nhopCount = 0;
	for (Packet *pp = 0, *po = Qnative_.head(); po; pp= po, po= po->next_) {
		u_int32_t prevHop = HDR_CMN(po)->prev_fwd_mac;
		if (prevHop == prev) nhopCount ++;
	}
	for (Packet *pp = 0, *po = Qoverheard_.head(); po; pp= po, po= po->next_) {
		u_int32_t prevHop = HDR_CMN(po)->prev_fwd_mac;
		if (prevHop == prev) nhopCount ++;
	}
	
	int nhopLim = Qnative_.limit()*DIVERSITY_FACTOR;
	if ( (nhopCount > nhopLim) ){
			return true;
	}
	return false;
}

bool Pri_sched::removeAcked(int uid, int acksndr) { //return true if entry is removed
	Packet *p = NULL;
	uid_pkt_tbl_t::iterator mp;
	mp = uid_pkt_map.find(uid);
	if (mp != uid_pkt_map.end()) {
		p = mp->second;
		u_int32_t p_nextH = (u_int32_t) ETHER_ADDR(HDR_MAC802_11(p)->dh_ra);
		if ( p_nextH == acksndr ) {
			Packet *tmp = removePktInQ(uid);
			drop(tmp, "ACK-RMVED");
			return true;
		}
	}
	return false;
}

Packet * Pri_sched::removePktInQ(int uid) {  //free packet after this call!!
	Packet *p = NULL;
	uid_pkt_tbl_t::iterator mp;			
	struct hdr_cmn *ch;
	mp = uid_pkt_map.find(uid);
	if (mp != uid_pkt_map.end()) {
		p = mp->second;
		ch = HDR_CMN(p);
		u_char q_belong = ch->qlocation;
		assert(q_belong != QLOC_UNQUED);

		if (q_belong == QLOC_NATIVE) 
			Qnative_.remove(p, p->prev_);
		else if (q_belong == QLOC_OVERHEARD)
			Qoverheard_.remove(p, p->prev_);
		else { // == QLOC_CODABLE
			assert(q_belong == QLOC_CODABLE);

			if ( hdr_cmn::access(ch->partner_p)->partner_p == p ) { // 2-partners
				Qcodable_.remove(p, p->prev_);
				Packet * ppartner = hdr_cmn::access(p)->partner_p;
				Qcodable_.remove(ppartner, ppartner->prev_);
				if (hdr_cmn::access(ppartner)->pcondition == PCOND_OVERHEARD) {
					Qnative_.enqueHead(ppartner);
					hdr_cmn::access(ppartner)->qlocation = QLOC_NATIVE;
				} else {
					Qoverheard_.enqueHead(ppartner);
					hdr_cmn::access(ppartner)->qlocation = QLOC_OVERHEARD;
				}
				hdr_cmn::access(ppartner)->partner_p = NULL;
			} else { // > 2 partners
				Packet * prev_p = 0;
				if ( p == Qcodable_.head() || hdr_cmn::access(p->prev_)->partner_p != p ) { // means p is head partner
					Packet *lastp;
					for (lastp=p; hdr_cmn::access(lastp)->partner_p != p; lastp = lastp->next_)
					{}
					prev_p = lastp;
				} else
					prev_p = p->prev_;
					
				hdr_cmn::access(prev_p)->partner_p = ch->partner_p;
				Qcodable_.remove(p, p->prev_);
			}	
		}
		hdr_cmn::access(p)->partner_p = NULL;
		hdr_cmn::access(p)->qlocation = QLOC_UNQUED;
		//uid_pkt_map.erase(mp);
	}	
	return p;
}

void
Pri_sched::queue_and_searchPartner(Packet *inp, bool istail) {
	struct hdr_cmn *ch_inp = HDR_CMN(inp);
//	assert(inp->prev_==0 && inp->next_==0);
	
	Packet * head_partner = 0;
	bool coding_search = true;
	
	uid_pkt_tbl_t::iterator mp = uid_pkt_map.find(ch_inp->uid());			
	if (mp != uid_pkt_map.end()) {
		//printf("Found duplicated packet in map and queue");
		drop(inp, "inQalready");
		return;
	}

	Packet *p = Qcodable_.head();
	for (Packet *pp = 0; p; pp= p, p= p->next_) {
		struct hdr_cmn *ccc = HDR_CMN(p);
		if (p == Qcodable_.head() || hdr_cmn::access(p->prev_)->partner_p != p)
			head_partner = p;
		while (ccc->partner_p != head_partner) { // end of the partner list
		 	if ( coding_search ) {
		 		if ( !(isPartner(p, inp)) ) coding_search = false;
			}
			pp = p;
			p = p->next_;
			ccc = hdr_cmn::access(p);
		} // it stops at the last partner
		if ( !(isPartner(p, inp)) ) coding_search = false;
		if (coding_search) {
			if ( Qcodable_.enque_insert(inp, p) ) {
				ch_inp->partner_p = hdr_cmn::access(p)->partner_p; // end of the coding list; point to head
				assert(ch_inp->partner_p == head_partner);
				//debug jz
				if (head_partner == Qcodable_.head()) {
//					printf("head-3 %d\n", Qcodable_.length());
				} else {
					printf("mid-3 %d\n", Qcodable_.length());
				}
				//debug jz
				
				hdr_cmn::access(p)->partner_p = inp;
				ch_inp->qlocation = QLOC_CODABLE;
				return;
			} else
				break; // if overflow, don't have to scan again;
		}
	}
	

	if (Qcodable_.isAdd2_Overflow(inp)) { // if Qcodable is going to overflow, stop coding
		if (ch_inp->pcondition == PCOND_OVERHEARD) {
			Qoverheard_.enque(inp);
			ch_inp->qlocation = QLOC_OVERHEARD;
		} else {
			Qnative_.enque(inp);
			ch_inp->qlocation = QLOC_NATIVE;
		}
		ch_inp->partner_p = NULL;
		return;
	}
	
		
	for (Packet *pp = 0, *p = Qnative_.head(); p; pp= p, p= p->next_) {
		if (isPartner(p, inp)) {
			if (!pp) Qnative_.deque();
			else
				Qnative_.remove(p, pp); // remove the packet following pp

			struct hdr_cmn *ch_p = HDR_CMN(p);
			Qcodable_.enque(p);
			ch_p->qlocation = QLOC_CODABLE;
			Qcodable_.enque(inp);
			ch_inp->qlocation = QLOC_CODABLE;
			ch_p->partner_p = inp;
			ch_inp->partner_p = p; // end of the coding list
			return;
		}
	}
	
	for (Packet *pp = 0, *p = Qoverheard_.head(); p; pp= p, p= p->next_) {
		if (isPartner(p, inp)) {
			if (!pp) Qoverheard_.deque();
			else
				Qoverheard_.remove(p, pp); // remove the packet following pp

			struct hdr_cmn *ch_p = HDR_CMN(p);
			Qcodable_.enque(p);
			ch_p->qlocation = QLOC_CODABLE;
			Qcodable_.enque(inp);
			ch_inp->qlocation = QLOC_CODABLE;
			ch_p->partner_p = inp;
			ch_inp->partner_p = p; // end of the coding list
			return;
		}
	}
		
	if (ch_inp->pcondition == PCOND_OVERHEARD) {
		Qoverheard_.enque(inp);
		ch_inp->qlocation = QLOC_OVERHEARD;
	} else {
		Qnative_.enque(inp);
		ch_inp->qlocation = QLOC_NATIVE;
	}
	ch_inp->partner_p = NULL;
}

bool Pri_sched::isPartner(Packet *p, Packet *pn) {
	struct hdr_cmn *ch_p = HDR_CMN(p);
	struct hdr_cmn *ch_pn = HDR_CMN(pn);
	
if ( ((ch_p->pcondition == PCOND_INTENDED_CODABLE) || (ch_p->pcondition == PCOND_OVERHEARD) || (ch_p->pcondition == PCOND_DECODED)) 
 && ((ch_pn->pcondition == PCOND_INTENDED_CODABLE) || (ch_pn->pcondition == PCOND_OVERHEARD) || (ch_pn->pcondition == PCOND_DECODED)) )
//	if ( (ch_p->pcondition != PCOND_ORIG) && (ch_pn->pcondition != PCOND_ORIG) )
	{
		bool p2pn = false, pn2p = false;
//		int n_hopmac_p = arptable_->arplookup_mac(ch_p->two_hop_away_ip);
//		int n_hopmac_pn = arptable_->arplookup_mac(ch_pn->two_hop_away_ip);

//		int n_hopmac_p = arptable_->arplookup_mac(ch_p->next_hop_);
//		int n_hopmac_pn = arptable_->arplookup_mac(ch_pn->next_hop_);
		int n_hopmac_p = ch_p->next_hop_;
		int n_hopmac_pn = ch_pn->next_hop_;
		
		if (n_hopmac_p == -1 || n_hopmac_pn == -1)
			return false;
		if ((n_hopmac_p == n_hopmac_pn) || (ch_pn->prev_fwd_mac == ch_p->prev_fwd_mac)) 
			return false;
//		neighbor_ent * ngb_nhop = arptable_->my_neighbors_.lookup_neighbor(n_hopmac_p);
//		if (ngb_nhop->next_neighbors->lookup_neighbor(n_hopmac_pn))
//			return false;
		//debug
		//int xxx =((Mac802_11 *) target())->addr();
		
// if p.2hp is pn.prevfwd or prevfwd's neighbor
		if (n_hopmac_p == ch_pn->prev_fwd_mac)
			p2pn = true;
		else {
			 if (ch_pn->pcondition != PCOND_DECODED) {
			 	neighbor_ent * ngb_prevfwd = arptable_->my_neighbors_.lookup_neighbor(ch_pn->prev_fwd_mac);
			 	assert(ngb_prevfwd);
			 	if (ngb_prevfwd->next_neighbors->lookup_neighbor(n_hopmac_p))
			 		p2pn = true;
			 }
		}
		if (p2pn) {
			// if pn.2hp is p.prevfwd or prevfwd's neighbor			 	
			if (n_hopmac_pn == ch_p->prev_fwd_mac)
				pn2p = true;
			else {
				if (ch_p->pcondition != PCOND_DECODED) {
				 	neighbor_ent * ngb_prevfwd = arptable_->my_neighbors_.lookup_neighbor(ch_p->prev_fwd_mac);
				 	assert(ngb_prevfwd);
				 	if (ngb_prevfwd->next_neighbors->lookup_neighbor(n_hopmac_pn))
					 	pn2p = true;
				}
			}
		}

		if (p2pn && pn2p)
			return true;
	
		//if ( (ch_p->pcondition == PCOND_OVERHEARD) || (ch_pn->pcondition == PCOND_OVERHEARD) ) {
			// WHich means BEND works here; count it later and compare with the total codings
		//}
	}	
	
	return false;
}

/*
 * Called at the end of the simulation to purge the IFQ.
 */
void
Pri_sched::Terminate()
{
	Packet *p;
	while ((p = Qbroadcast_.deque())) {
	    drop(p, DROP_END_OF_SIMULATION);
	}
	
    while((p = Qnative_.deque())) {
		    drop(p, DROP_END_OF_SIMULATION);
    }
    
    while((p = Qoverheard_.deque())) {
		    drop(p, DROP_END_OF_SIMULATION);
    }
       
    while((p = Qcodable_.deque())) {
		    drop(p, DROP_END_OF_SIMULATION);
    }
    
}

void Pri_sched::resume() {
	sched();
}

void Pri_sched::sched() {
	Packet * p;
	blocked_ = 0;
// 1st priority: Broadcasting and some routing msg	
	if ( Qbroadcast_.length() > 0 ) {
		p = Qbroadcast_.deque();
		assert(p != 0);
		blocked_ = 1;
		((Mac802_11 *) target())->set_currtxm_coded(false);
		((Mac802_11 *) target())->codingPair_[0] = 0;
		((Mac802_11 *) target())->codingPair_[1] = 0;
		target_->recv(p, &qh_);
		return;
	} 

	if (disable_coding) {
		double intended_w = Random::uniform(1.0);
		double overheard_w = Random::uniform(OVERHEARD_Q_WEIGHT_OPPFWD);
		
		if ( intended_w >= overheard_w ) {
			if (Qnative_.length() > 0) 
				schedNative();
			else if (Qoverheard_.length() > 0)
				schedOverheard();
		} else {
			if (Qoverheard_.length() > 0) 
				schedOverheard();
			else if (Qnative_.length() > 0)
				schedNative();
		}
		
		return;		
	}
// 
//	double native_w = Random::uniform(NATIVE_Q_WEIGHT * (Qnative_.length()));
//	double codable_w = Random::uniform(1.0 * (Qcodable_.length()));
	double native_w = Random::uniform(NATIVE_Q_WEIGHT);
	double codable_w = Random::uniform(Qcodable_weight);

	if (native_w >= codable_w) {
		if (Qnative_.length() > 0) 
			schedNative();
		else if (Qcodable_.length() > 0)
			schedCoded();
	} else {
		if (Qcodable_.length() > 0) 
			schedCoded();
		else if (Qnative_.length() > 0)
			schedNative();
	}
	resetQcodeOnce();
	
}

void Pri_sched::schedNative() {
	
		if (Qcodable_.length() == 0 && Qnative_.length() <= 0) // 20)
			return; // keep the queue length to 20
	
		Packet *p = Qnative_.deque();
		assert(p != 0);
		
//		//debug
//		if (HDR_CMN(p)->pcondition == PCOND_OVERHEARD) {
//			drop(p, "");
//			return;
//		}
		
		
		blocked_ = 1;
		((Mac802_11 *) target())->set_currtxm_coded(false);
//		struct hdr_cmn *hdr = HDR_CMN(p); //debug jz
//		double now = Scheduler::instance().clock(); // debug jz
//		if (hdr->uid() == 44 || hdr->uid() == 40) {
//			printf("IFQ-N--%f- I-%d saw %d %d; \n", now, 
//			((Mac802_11 *) target())->addr(), hdr->uid(), hdr->ptype());
//			
//		} //debug jz
		
		((Mac802_11 *) target())->codingPair_[0] = 0;
		((Mac802_11 *) target())->codingPair_[1] = 0;
		target_ -> recv(p, &qh_);	
}


void Pri_sched::schedOverheard() {
	
		Packet *p = Qoverheard_.deque();
		assert(p != 0);
		
		blocked_ = 1;
		((Mac802_11 *) target())->set_currtxm_coded(false);
		
		((Mac802_11 *) target())->codingPair_[0] = 0;
		((Mac802_11 *) target())->codingPair_[1] = 0;
		target_ -> recv(p, &qh_);	
}

void Pri_sched::schedCoded() {
	
	int count = 0;
	Packet *p = 0, *pp = 0;
	Packet *head = Qcodable_.head();

	do {
		p = Qcodable_.deque();
		assert(count < MAX_CODING_LEN);
		((Mac802_11 *) target())->codingPair_[count] = p;
		assert(p != 0);
		assert(pp == 0 || hdr_cmn::access(pp)->partner_p == p);
		count ++;
		pp = p;
	} 	while ( hdr_cmn::access(p)->partner_p != head );
	
	((Mac802_11 *) target())->set_currtxm_coded(true);
	((Mac802_11 *) target())->codingCount_ = count;
	
	blocked_ = 1;
	target_ -> recv(head, &qh_);	
}
	
void qlenTr_timer::expire (Event * e)
{
	agent_->printQlen();
	agent_->reset_qlenTr_timer();
}

void Pri_sched::printQlen() {
	double now = Scheduler::instance().clock();
	if (t_ch)
		qlentrace("-Nd %d	%f	-Native %d	-Ovhrd %d	-Codable %d\n", 
				((Mac802_11 *) target())->addr(), now, Qnative_.length(), Qoverheard_.length(), Qcodable_.length());
}

void Pri_sched::reset_qlenTr_timer ()
{
        qlenTimer_.resched (qlen_trace_prd_);        // need to adjust this parameter later
}

void Pri_sched::qlentrace(char* fmt, ...)
{
	char buffer[256];
	va_list ap;
	va_start(ap, fmt);
	vsprintf(buffer, fmt, ap);
	(void)Tcl_Write(t_ch, buffer, strlen(buffer));
	va_end(ap);
}


