/***************************************************************************
 *            Pri_sched.h
 *
 *  Mon Mar  6 22:49:04 2006
 *  Copyright  2006  User
 *  Email
 ****************************************************************************/

#ifndef Pri_sched_h
#define Pri_sched_h

#include "queue.h"
#include "Pri_dtail.h"
#include "packet.h"
#include "lib/bsd-list.h"
#include "arp.h"

#define MAX_CODING_LEN 8

#define DIVERSITY_FACTOR 0.2 //0.2 // i.e., at most 1/5 of the queue size can be occupied by packets to the same next hop.
#define CODING_SIZE_THRHLD 300
#define NATIVE_Q_WEIGHT 0.1
#define QCODABLE_DEFAULT_W 1.0

#define OVERHEARD_Q_WEIGHT 0
#define OVERHEARD_Q_WEIGHT_OPPFWD 0.3

class Pri_sched;

typedef int (*PacketFilter)(Packet *, void *);

LIST_HEAD(Pri_sched_List, Pri_sched);



class qlenTr_timer : public TimerHandler {
public:
        qlenTr_timer(Pri_sched* agent) : TimerHandler() {
                agent_ = agent;
        }
protected:
        Pri_sched* agent_;
        virtual void expire(Event* e);
};


class Pri_sched : public Pri_dtail {
	friend class qlenTr_timer;
public:
    Pri_sched();
	~Pri_sched() {
	    Terminate();
	}
    int     command(int argc, const char*const* argv);
    void    recv(Packet *p, Handler *h);
    void    pri_recv(Packet *p, Handler *h);
    void    first_pri_recv(Packet *p, Handler *h);
    
    Pri_dtail Qnative_;
    Pri_dtail Qoverheard_;
    Pri_dtail Qcodable_;
    Pri_dtail Qbroadcast_;
	uid_pkt_tbl_t uid_pkt_map;
			
    void    Terminate(void);
    
	void sched();
	virtual void resume();
	void setPacketLeft(Packet * p);
	
	void printQlen();
	void qlentrace(char* fmt, ...);
	void reset_qlenTr_timer();

	void queue_and_searchPartner(Packet *p, bool istail);
	Packet * removePktInQ(int uid);
	bool removeAcked(int uid, int acksndr);
	bool isPartner(Packet *p1, Packet *p2);
	bool isCodingDisabled() { return disable_coding; };
	inline void disableQcodeOnce() { Qcodable_weight = 0.0; };
	inline void resetQcodeOnce() { Qcodable_weight = QCODABLE_DEFAULT_W; };
	bool activeMacDrop(u_int32_t prev);
	bool activeMacDrop_overheard(u_int32_t prev);
protected:
	qlenTr_timer qlenTimer_;
  	ARPTable*  arptable_;

  	void schedNative();
	void schedCoded();
	void schedOverheard();
  	
private:
    int Prefer_Routing_Protocols;
    int cntl;
	int flag;
	bool disable_coding;
	Packet * packet_left_;

	double qlen_trace_prd_;
	Tcl_Channel     t_ch;
	
	float Qcodable_weight;

        /*
	 * A global list of Interface Queues.  I use this list to iterate
	 * over all of the queues at the end of the simulation and flush
	 * their contents. - josh
	 */
public:
    LIST_ENTRY(Pri_sched) link;
    static struct Pri_sched_List prhead;
};

#endif
