# run under test dir
# /home/jianz/ns-allinone-2.28/ns-2.28_deflect/ns /home/jianz/ns-allinone-2.28/test/tcl_file/deflect.tcl  1Mb  /home/jianz/ns2sim_data_file/deflect

#debug 1

if { $argc != 4 } {
   puts "usage error!"
   exit
}	

set val(seed)		[lindex $argv 2];
set val(protocol)		[lindex $argv 3]; # COPE or BEND or 802.11
set val(rate)		[lindex $argv 0];
set val(rthd)		3000; # always disable RTS/CTS
set val(path)		[lindex $argv 1];
set val(srcpath)	/home/jianz/ns-allinone-2.28/test/tcl_file  ;# working path
set val(chan)           Channel/WirelessChannel    ;#Channel Type
set val(netif)          Phy/WirelessPhy            ;# network interface type
set val(mac)            Mac/802_11                 ;# MAC type
set val(ifq)		Queue/DropTail/PriQueue		;
set val(ll)             LL                         ;# link layer type
set val(ant)            Antenna/OmniAntenna        ;# antenna model
set val(ifqlen)         100                         ;# set length of Qnative_ and Qcodable
set val(nn)             25; #15; #10; #14; #7; #12; #10; #13		          ;# number of mobilenodes
set val(stopat)		320; 

set val(prop)           Propagation/TwoRayGround   ;# radio-propagation model
#set val(prop)           Propagation/Ricean         ;# Rayleigh and Ricean
#set val(RiceanK)        6                          ;# Ricean K factor
#set val(RiceanMaxVel)   0.5                        ;# Ricean  Propagation  MaxVelocity Parameter
# Ricean  Propagation: Maximum ID of nodes (Total number of nodes) used to
# compute pairwise table offsets.
set val(RiceMaxNodeID)  [expr {$val(nn)-1}]        ;
set val(RiceDataFile)   $val(srcpath)/rice_table.txt             ;# Ricean Propagation Data File
set val(proplog)        $val(path)/ricean-proplog.tr   ;

# routing protocol
#set val(rp)              DumbAgent  
set val(rp)             DSDV                     

set val(x)		1100
set val(y)		1100

#set val(seed) 105;
ns-random $val(seed);


ErrorModel80211 noise1  -104                           ;# 1Mbps datarate thermal noise
ErrorModel80211 noise2  -104
ErrorModel80211 shortpreamble	0                       ;# toggle 802.11 short preamble on/off
#ErrorModel80211 LoadBerSnrFile $val(srcpath)/ber_snr_intersil.txt    ;# use Intersil BER/SNR empirical curves
ErrorModel80211 LoadBerSnrFile $val(srcpath)/ber_snr_choi.txt         ;# use theoretical curves from del Prado
                                                        ;# Pavon and Choi, "Link Adaptation Strategy
                                                        ;# for IEEE 802.11 WLAN via Received Signal
                                                        ;# Strength Measurement"

remove-all-packet-headers       ;# removes all except common
add-packet-header IP CBR ARP LL Mac Message TCP

# Initialize Global Variables
set ns_		[new Simulator]
set tracefd     [open $val(path)/deflect.tr w]
$ns_ use-newtrace
$ns_ trace-all $tracefd

#set namtrace [open $val(path)/deflect.nam w]
#$ns_ namtrace-all-wireless $namtrace $val(x) $val(y)

set loadTrace	[open $val(path)/deflect.lt w]
set qlenTrace	[open $val(path)/deflect.ql w]


# set up topography object
set topo       [new Topography]

$topo load_flatgrid $val(x) $val(y)

# Create God
create-god $val(nn)

# Create channel
set chan_ [new $val(chan)]

#802.11b
Mac/802_11 set basicRate_ 1Mb; #Rate for Control Frames
Mac/802_11 set PLCPDataRate_      1Mb             ;# 1Mbps
Mac/802_11 set dataRate_         $val(rate)       ;
Mac/802_11 set RTSThreshold_ $val(rthd)    ;# turn on RTS/CTS

#Phy/WirelessPhy set CPThresh_ 12.0;
#Phy/WirelessPhy set CSThresh_ 1.559e-12;

# Set MAC Medium load parameters
Mac/802_11 set scan_len_    1000     ;# scan count for each probe
Mac/802_11 set scan_int_    0.001	;# 0.0001   ;# scanning interval // disable by large #
Mac/802_11 set smooth_scan_ 1   ;# smooth the scanned values

Mac/802_11 set link_ber_ 2e-6;

#Queue set limit_ 150  ;# set Qcodable and Qnative length

#Agent/DSDV set perup_  100;
#Agent/DSDV set verbose_ 1;
#Agent/DSDV set update_twice_ 1;

$ns_ node-config -adhocRouting $val(rp) \
		-llType $val(ll) \
		-macType $val(mac) \
		-ifqType $val(ifq) \
		-ifqLen $val(ifqlen) \
		-antType $val(ant) \
		-propType $val(prop) \
		-phyType $val(netif) \
		-topoInstance $topo \
		-agentTrace ON \
		-routerTrace ON \
		-macTrace ON \
		-movementTrace OFF \
		-channel $chan_

# Set propagation settings
if { $val(prop) == "Propagation/Ricean"} {
    set prop_inst [$ns_ set propInstance_]
    #$prop_inst MaxVelocity  2.5;
    #$prop_inst RiceanK        6;
    #$prop_inst LoadRiceFile  "rice_table.txt";

    $prop_inst MaxVelocity  $val(RiceanMaxVel);
    $prop_inst RiceanK      $val(RiceanK);
    $prop_inst LoadRiceFile  $val(RiceDataFile);
    $prop_inst RiceMaxNodeID $val(RiceMaxNodeID);

    ###############################################
    #   LOG the propagation information
    ## ###########################################
    if { $val(proplog) != "null" } {
	set prop_tracefd [open $val(proplog) w];
	set prop_log [new BaseTrace]
	$prop_log attach $prop_tracefd
	$prop_log set src_ 0
	$prop_inst tracetarget $prop_log
    }
}

for {set i 0} {$i < $val(nn)} {incr i} {
    set node_($i) [$ns_ node]
    $node_($i) random-motion 0
}

#
# Provide initial (X,Y, for now Z=0) co-ordinates for mobilenodes
#source $val(srcpath)/nodeloc_deflect_simple.tcl
#source $val(srcpath)/nodeloc_deflect_drum.tcl
#source $val(srcpath)/nodeloc_deflect_grid.tcl
#source $val(srcpath)/nodeloc_deflect_831.tcl
#source $val(srcpath)/nodeloc_deflect_821.tcl
#source $val(srcpath)/nodeloc_deflect_grid_2x5.tcl
#source $val(srcpath)/nodeloc_deflect_chain-4.tcl
#source $val(srcpath)/nodeloc_deflect_grid_3x5.tcl
source $val(srcpath)/nodeloc_deflect_grid_5x5.tcl

for {set i 0} {$i < $val(nn)} {incr i} {
    $ns_ initial_node_pos $node_($i) 20
}

# set the special cw size for gateway;
#[$node_(0) set mac_(0)] set-cw-orig 16 


source $val(srcpath)/traffic_gen_lib.tcl

#source $val(srcpath)/cbr_traffic_deflect_simple.tcl
#source $val(srcpath)/cbr_traffic_deflect_drum.tcl
#source $val(srcpath)/cbr_traffic_deflect_831.tcl
#source $val(srcpath)/cbr_traffic_deflect_821.tcl
#source $val(srcpath)/cbr_traffic_deflect_grid_2x5.tcl
#source $val(srcpath)/cbr_traffic_deflect_chain-4.tcl
#source $val(srcpath)/cbr_traffic_deflect_grid_3x5.tcl
source $val(srcpath)/cbr_traffic_deflect_grid_5x5.tcl

#$ns_ at 208.19274744 "[$node_(6) set ragent_] dumprtab";
#$ns_ at 208.201403981 "[$node_(6) set ragent_] dumprtab";

#$ns_ at 200 "[$node_(4) set ragent_] dumprtab";

#$ns_ at 320 "[$node_(0) set ifq_(0)] debug-qlen-chk";
#$ns_ at 188.95 "[$node_(1) set ifq_(0)] debug-qlen-chk";
#$ns_ at 188.95 "[$node_(2) set ifq_(0)] debug-qlen-chk";
#$ns_ at 188.95 "[$node_(3) set ifq_(0)] debug-qlen-chk";
#$ns_ at 188.95 "[$node_(4) set ifq_(0)] debug-qlen-chk";
#$ns_ at 395 "[$node_(5) set ifq_(0)] debug-qlen-chk";
#$ns_ at 395 "[$node_(6) set ifq_(0)] debug-qlen-chk";
#$ns_ at 395 "[$node_(7) set ifq_(0)] debug-qlen-chk";
#$ns_ at 395 "[$node_(8) set ifq_(0)] debug-qlen-chk";
#$ns_ at 395 "[$node_(9) set ifq_(0)] debug-qlen-chk";


#$ns_ at 200 "[$node_(3) set ifq_(0)] debug-qlen-chk";

#$ns_ at 350 "[$node_(6) set ifq_(0)] debug-qlen-chk";
#$ns_ at 350 "[$node_(1) set ifq_(0)] debug-qlen-chk";
#$ns_ at 350 "[$node_(8) set ifq_(0)] debug-qlen-chk";
$ns_ at 300 "[$node_(6) set ifq_(0)] debug-q-content";

#$ns_ at 188.95 "[$node_(0) set mac_(0)] debug-qlen-chk";
#$ns_ at 270 "[$node_(8) set mac_(0)] debug-q-content";

#$ns_ at 400 "[$node_(0) set mac_(0)] print-txm-count";
$ns_ at 320 "[$node_(1) set mac_(0)] print-txm-count";
$ns_ at 320 "[$node_(2) set mac_(0)] print-txm-count";
#$ns_ at 320 "[$node_(3) set mac_(0)] print-txm-count";
#$ns_ at 400 "[$node_(4) set mac_(0)] print-txm-count";
#$ns_ at 400 "[$node_(5) set mac_(0)] print-txm-count";
$ns_ at 320 "[$node_(6) set mac_(0)] print-txm-count";
$ns_ at 320 "[$node_(7) set mac_(0)] print-txm-count";
#$ns_ at 320 "[$node_(8) set mac_(0)] print-txm-count";
#$ns_ at 400 "[$node_(9) set mac_(0)] print-txm-count";
#$ns_ at 400 "[$node_(10) set mac_(0)] print-txm-count";
#$ns_ at 400 "[$node_(11) set mac_(0)] print-txm-count";

#$ns_ at 400 "[$node_(5) set mac_(0)] print-txm-count";
#$ns_ at 400 "[$node_(5) set mac_(0)] print-txm-count";
#$ns_ at 400 "[$node_(10) set mac_(0)] print-txm-count";
#$ns_ at 400 "[$node_(11) set mac_(0)] print-txm-count";
#$ns_ at 400 "[$node_(12) set mac_(0)] print-txm-count";
#$ns_ at 400 "[$node_(13) set mac_(0)] print-txm-count";

#$ns_ at 400 "[$node_(3) set mac_(0)] print-txm-count";


#
# Tell nodes when the simulation ends
#
for {set i 0} {$i < $val(nn) } {incr i} {
    $ns_ at $val(stopat) "$node_($i) reset";
}
$ns_ at $val(stopat) "stop"
$ns_ at $val(stopat)+0.01 "puts \"NS EXITING...\" ; $ns_ halt"
proc stop {} {
    global ns_ tracefd
    $ns_ flush-trace
    close $tracefd
}


#new MashInspector

puts "Starting Simulation..."
$ns_ run


