if { $argc != 2 } {
   puts "usage error!"
   exit
}	

set val(rate)		[lindex $argv 0];
set val(rthd)		3000; # always enable RTS/CTS
set val(path)		[lindex $argv 1];
set val(srcpath)	/home/jianz/ns-allinone-2.28/test/tcl_file  ;# working path
set val(chan)           Channel/WirelessChannel    ;#Channel Type
set val(netif)          Phy/WirelessPhy            ;# network interface type
set val(mac)            Mac/802_11                 ;# MAC type
set val(ifq)            Queue/DropTail/PriQueue   ;# interface queue type
set val(ll)             LL                         ;# link layer type
set val(ant)            Antenna/OmniAntenna        ;# antenna model
set val(ifqlen)         300                         ;# max packet in ifq
set val(nn)             5; #7; #7; #13		          ;# number of mobilenodes
set val(stopat)		400; #; #700;  #150;  

set val(prop)           Propagation/TwoRayGround   ;# radio-propagation model
#set val(prop)           Propagation/Ricean         ;# Rayleigh and Ricean
set val(RiceanK)        6                          ;# Ricean K factor
set val(RiceanMaxVel)   0.1                        ;# Ricean  Propagation  MaxVelocity Parameter
# Ricean  Propagation: Maximum ID of nodes (Total number of nodes) used to
# compute pairwise table offsets.
set val(RiceMaxNodeID)  [expr {$val(nn)-1}]        ;
set val(RiceDataFile)   $val(srcpath)/rice_table.txt             ;# Ricean Propagation Data File
set val(proplog)        $val(path)/ricean-proplog.tr   ;

ErrorModel80211 noise1  -104                            ;# 1Mbps datarate thermal noise
ErrorModel80211 noise2  -104
ErrorModel80211 shortpreamble	0                       ;# toggle 802.11 short preamble on/off
ErrorModel80211 LoadBerSnrFile $val(srcpath)/ber_snr_intersil.txt    ;# use Intersil BER/SNR empirical curves
#ErrorModel80211 LoadBerSnrFile ber_snr_choi.txt         ;# use theoretical curves from del Prado
                                                        ;# Pavon and Choi, "Link Adaptation Strategy
                                                        ;# for IEEE 802.11 WLAN via Received Signal
                                                        ;# Strength Measurement"

# routing protocol
#set val(rp)              DumbAgent  
set val(rp)             DSDV                     

set val(x)		1100
set val(y)		1100

set val(seed) 100;
ns-random $val(seed)

remove-all-packet-headers       ;# removes all except common
add-packet-header IP CBR ARP LL Mac Message TCP

# Initialize Global Variables
set ns_		[new Simulator]
set tracefd     [open $val(path)/maca_lega.tr w]
$ns_ use-newtrace
$ns_ trace-all $tracefd

#set namtrace [open $val(path)/lega.nam w]
#$ns_ namtrace-all-wireless $namtrace $val(x) $val(y)

#set loadTrace	[open $val(path)/lega.lt w]
#set qlenTrace	[open $val(path)/lega.ql w]


# set up topography object
set topo       [new Topography]

$topo load_flatgrid $val(x) $val(y)

# Create God
create-god $val(nn)

# Create channel
set chan_ [new $val(chan)]

#Phy/WirelessPhy set CSThresh_ 3.652e-10
#Phy/WirelessPhy set RXThresh_ 3.652e-10

#802.11b
Mac/802_11 set basicRate_ 1Mb; #Rate for Control Frames
Mac/802_11 set PLCPDataRate_      1Mb             ;# 1Mbps
Mac/802_11 set dataRate_         $val(rate)       ;
Mac/802_11 set RTSThreshold_ $val(rthd)    ;# turn on RTS/CTS

# Set MAC Medium load parameters
Mac/802_11 set scan_len_    10000     ;# scan count for each probe
Mac/802_11 set scan_int_    1000  ;# scanning interval
Mac/802_11 set smooth_scan_ 1   ;# smooth the scanned values


#Agent/DSDV set perup  50;

$ns_ node-config -adhocRouting $val(rp) \
		-llType $val(ll) \
		-macType $val(mac) \
		-ifqType $val(ifq) \
		-ifqLen $val(ifqlen) \
		-antType $val(ant) \
		-propType $val(prop) \
		-phyType $val(netif) \
		-topoInstance $topo \
		-agentTrace ON \
		-routerTrace ON \
		-macTrace ON \
		-movementTrace OFF \
		-channel $chan_

# Set propagation settings
if { $val(prop) == "Propagation/Ricean"} {
    set prop_inst [$ns_ set propInstance_]
    #$prop_inst MaxVelocity  2.5;
    #$prop_inst RiceanK        6;
    #$prop_inst LoadRiceFile  "rice_table.txt";

    $prop_inst MaxVelocity  $val(RiceanMaxVel);
    $prop_inst RiceanK      $val(RiceanK);
    $prop_inst LoadRiceFile  $val(RiceDataFile);
    $prop_inst RiceMaxNodeID $val(RiceMaxNodeID);

    ###############################################
    #   LOG the propagation information
    ## ###########################################
    if { $val(proplog) != "null" } {
	set prop_tracefd [open $val(proplog) w];
	set prop_log [new BaseTrace]
	$prop_log attach $prop_tracefd
	$prop_log set src_ 0
	$prop_inst tracetarget $prop_log
    }
}


for {set i 0} {$i < $val(nn)} {incr i} {
    set node_($i) [$ns_ node]
    $node_($i) random-motion 0
}

#
# Provide initial (X,Y, for now Z=0) co-ordinates for mobilenodes
source $val(srcpath)/nodeloc_deflect_simple.tcl
#source $val(srcpath)/nodeloc_deflect_drum.tcl
#source $val(srcpath)/nodeloc_deflect_grid.tcl
#source $val(srcpath)/nodeloc_deflect_821.tcl
#source $val(srcpath)/nodeloc_deflect_grid_2x5.tcl

for {set i 0} {$i < $val(nn)} {incr i} {
    $ns_ initial_node_pos $node_($i) 20
   # [$node_($i) set mac_(0)] load-trace $loadTrace
}

source $val(srcpath)/traffic_gen_lib.tcl
source $val(srcpath)/cbr_traffic_deflect_simple.tcl
#source $val(srcpath)/cbr_traffic_deflect_drum.tcl
#source $val(srcpath)/cbr_traffic_deflect_821.tcl
#source $val(srcpath)/cbr_traffic_deflect_grid_2x5.tcl


#
# Tell nodes when the simulation ends
#
for {set i 0} {$i < $val(nn) } {incr i} {
    $ns_ at $val(stopat) "$node_($i) reset";
}
$ns_ at $val(stopat) "stop"
$ns_ at $val(stopat)+0.01 "puts \"NS EXITING...\" ; $ns_ halt"
proc stop {} {
    global ns_ tracefd
    $ns_ flush-trace
    close $tracefd
}


#new MashInspector

puts "Starting Simulation..."
$ns_ run


